@@ -1,17 +1,6 @@
 import emoji
 import re
 
-extra_args = {
-    "vinai/bertweet-base":  {
-        "user_token": "@USER",
-        "url_token": "HTTPURL",
-    }
-}
-
-
-
-special_tokens = ["@usuario", "url", "hashtag", "emoji"]
-
 
 user_regex = re.compile(r"@[a-zA-Z0-9_]{0,15}")
 url_regex = re.compile(
@@ -21,6 +10,7 @@ url_regex = re.compile(
 hashtag_regex = re.compile(r'\B#(\w*[a-zA-Z]+\w*)')
 start_of_camel = re.compile(r'([A-Z]+)')
 
+
 def camel_to_human(s, lower=True):
     """
     Converts camel case to 'human' case
@@ -39,12 +29,16 @@ def camel_to_human(s, lower=True):
 
     return ret
 
+
 emoji_regex = re.compile(r"\|([^\|]+)\|")
 
-def convert_emoji_to_text(x, emoji_wrapper="[EMOJI]"):
+
+def convert_emoji_to_text(x, emoji_wrapper="emoji"):
     """
+    Converts emoji to text
     """
-    return f" {emoji_wrapper} " + " ".join(x.groups()[0].split("_")) + f" {emoji_wrapper} "
+    wrapper = f" {emoji_wrapper} ".replace("  ", " ")
+    return wrapper + " ".join(x.groups()[0].split("_")) + wrapper
 
 
 replacements = {
@@ -101,14 +95,69 @@ laughter_conf = {
     "en": {
         "regex": re.compile("[ha][ha]+ah[ha]+"),
         "replacement": "haha",
+    },
+    "it": {
+        "regex": re.compile("[ha][ha]+ah[ha]+"),
+        "replacement": "haha",
+    },
+
+    "pt": {
+        "regex": re.compile("[ha][ha]+ah[ha]+|kk+"),
+        "replacement": "kk",
+    }
+}
+
+default_args = {
+    "es": {
+        "user_token": "@usuario",
+        "url_token": "url",
+        "hashtag_token": "hashtag",
+    },
+
+    "en": {
+        "user_token": "@USER",
+        "url_token": "HTTPURL",
+        "hashtag_token": "hashtag",
+    },
+
+    "it": {
+        "user_token": "##user",
+        "url_token": "##url",
+        "hashtag_token": "##hashtag",
+    },
+
+    "pt": {
+        "user_token": "@USER",
+        "url_token": "HTTPURL",
+        "hashtag_token": "hashtag",
     }
 }
 
 
+special_tokens = {
+    k: list(v.values()) for k, v in default_args.items()
+}
+
+model_preprocessing_args = {
+    "pysentimiento/robertuito-base-uncased": default_args["es"],
+    "pysentimiento/robertuito-base-cased": default_args["es"],
+    "pysentimiento/robertuito-base-deacc": default_args["es"],
+    "melll-uff/bertweetbr": default_args["en"],  # Same as BERTweet
+}
+
+
+def get_preprocessing_args(model_name, lang):
+    """
+    Gets preprocessing args for model_name and lang
+
+    Returns default_args[lang] if model_name has not special params
+    """
+    return model_preprocessing_args.get(model_name, default_args[lang])
+
 
 def preprocess_tweet(
-    text, lang="es", user_token="@usuario", url_token="url", preprocess_hashtags=True, hashtag_token=None,
-    demoji=True, shorten=3, normalize_laughter=True, emoji_wrapper="emoji"):
+        text, lang="es", user_token=None, url_token=None, preprocess_hashtags=True, hashtag_token=None, char_replace=True,
+        demoji=True, shorten=3, normalize_laughter=True, emoji_wrapper="emoji", preprocess_handles=True):
     """
     Basic preprocessing
 
@@ -136,35 +185,63 @@ def preprocess_tweet(
     shorten: int (default: 3)
         If not none, all occurrences of shorten or more characters are cut to this number
 
+    char_replace: bool (default: True)
+        If true, replaces or removes special characters to equivalent ones.
+
     demoji: boolean (default True)
-        If true, converts emoji to text representations using `emoji` library, and wraps this with "[EMOJI]" strings
+        If true, converts emoji to text representations using `emoji` library, and wraps this with "emoji" tokens
 
     normalize_laughter: boolean (default True)
         Normalizes laughters. Uses different regular expressions depending on the lang argument.
+
+    preprocess_handles: boolean (default True)
+        If true, replaces user handles with user_token
     """
-    if lang == "en" and user_token == "@usuario":
+
+    user_token = user_token or default_args[lang]["user_token"]
+    url_token = url_token or default_args[lang]["url_token"]
+    hashtag_token = hashtag_token or default_args[lang]["hashtag_token"]
+
+    def process_hashtags(x):
         """
-        If it is english and we didn't set any defaults, we set the vinai/bertweet-base defaults
+        Hashtag preprocessing function
+
+        Take first group and decamelize
         """
-        user_token = "@USER"
-        url_token = "HTTPURL"
 
+        text = x.groups()[0]
 
-    ret = ""
-    for char in text:
-        if char in replacements:
-            replacement = replacements[char]
-            if replacement:
-                ret += replacement
-        else:
-            ret += char
-    text = ret
+        text = camel_to_human(text)
+
+        if hashtag_token:
+            text = hashtag_token + " " + text
+
+        return text
+
+    if preprocess_hashtags:
+        text = hashtag_regex.sub(
+            process_hashtags,
+            text
+        )
+
+    if char_replace:
+        ret = ""
+        for char in text:
+            if char in replacements:
+                replacement = replacements[char]
+                if replacement:
+                    ret += replacement
+            else:
+                ret += char
+        text = ret
+
+    if preprocess_handles:
+        text = user_regex.sub(user_token, text)
 
-    text = user_regex.sub(user_token, text)
     text = url_regex.sub(url_token, text)
 
     if shorten:
-        repeated_regex = re.compile(r"(.)"+ r"\1" * (shorten-1) + "+")
+        repeated_regex = re.compile(r"(.)" + r"\1" * (shorten-1) + "+")
         text = repeated_regex.sub(r"\1"*shorten, text)
 
     if demoji:
@@ -183,27 +260,4 @@ def preprocess_tweet(
             text
         )
 
-    def process_hashtags(x):
-        """
-        Hashtag preprocessing function
-
-        Take first group and decamelize
-        """
-
-
-        text = x.groups()[0]
-
-        text = camel_to_human(text)
-
-        if hashtag_token:
-            text = hashtag_token + " " + text
-
-        return text
-
-    if preprocess_hashtags:
-        text = hashtag_regex.sub(
-            process_hashtags,
-            text
-        )
-
     return text.strip()
